// 此JS脚本为JS-SDK中间层。负责协调Native层和JS层的交互。
// @author Clare Qin
// @since 2016/5/18
(function () {

    var VERSION_CODE = 100;
    var VERSION_NAME = "V1.0";

    //防止重复创建。顺带判断版本。
    if (window.YmmBridge && window.YmmBridge.VERSION_CODE >= VERSION_CODE ) {
        return;
    }

    var _readyMessageIframe, _waiting_send_queue = [];
    var YMM_COMMON_PREFIX = "ymm-bridge:" + "/" + "/";

    var INIT_METHOD = "system.verify",init_callback_id;
    var _isValidated = false;
    var _isDebugMode = false;

    //事件绑定回调MAP
    var _event_bind_map = {};
    //调用返回回调MAP
    var _callback_map = {};

    function _log(tag,msg){
        if(_isDebugMode){
           console.log(tag+":"+msg);
        }
    }

    function _subscribe_event(event, callback) {
        //未验证，则不做操作。
        if(!_isValidated){
            return;
        }
        if (!event || typeof event !== 'string') {
            return;
        }
        if (typeof callback !== 'function') {
            return;
        }
        _event_bind_map[event] = callback;
    }

    function _unsubscribe_event(event){
        //未验证，则不做操作。
        if(!_isValidated){
            return;
        }
        if(!event || typeof event !== 'string'){
            return;
        }
        if(_event_bind_map[event]){
            delete _event_bind_map.event;
        }
    }


    function _notify_event(event,data) {
        var callback = _event_bind_map[event];
        if (!callback || typeof callback !== 'function') {
            return;
        } else {
            callback(data);
        }

    }

    //web端调用的通用接口
    function _call(method_name, params, callback) {
        if(!_isValidated && method_name !== INIT_METHOD){
            return;
        }
        _log("_call","method_name:"+method_name+",params:"+JSON.stringify(params)+",callback:"+callback);

        if (!callback || typeof callback !== 'function') {
            _log("_call","callback is not Function!->"+(typeof callback));
            return;
        }

        var callback_id = uuid();

        if(method_name === INIT_METHOD){
            init_callback_id = callback_id;
        }

        var waiting_request = {
            callback_id: callback_id,
            data: {
                method_name: method_name,
                params: params,
                callback: callback
            }
        };

        //设置接收结果的回调
        _callback_map[callback_id] = callback;

        //如果发送等待队列为空，则先直接发送。
        if (_waiting_send_queue.length <= 0) {
            _send_request(method_name, callback_id, params);
        }
        //将消息推入等待队列
        _waiting_send_queue.push(waiting_request);

    }

    //发送请求
    function _send_request(method_name, callback_id, params) {
        //构建请求对象。
        _log("_send_request","send REQUEST:"+method_name);
        var request_params = {callback_id: callback_id, data: params};
        method_name = method_name.replace(".", "/");
        var uri = YMM_COMMON_PREFIX + method_name + "?params=" + encodeURIComponent( JSON.stringify(request_params));
        if (!_readyMessageIframe) {
            _createQueueReadyIframe(document);
        }
        _readyMessageIframe.src = uri;
    }

    function _call_reply_from_native(callback_id) {
        _log("_call_reply_from_native","id:"+callback_id);
        var request;
        do {
            //找到当前的请求的ID，并将其从队列中去除。
            request = _waiting_send_queue.shift();
        } while (request.callback_id !== callback_id && _waiting_send_queue.length > 0);

        if (_waiting_send_queue.length > 0) {
            //如果队列中仍然有未发送请求，则继续发送。
            request = _waiting_send_queue[0];
            _send_request(request.data.method_name,
                request.callback_id,
                request.data.params);
        } else {
            //等待队列为空。
        }
    }

    function _notify_result(callback_id,result_resp){
        _log("notify_result:","callback_id:"+callback_id+",resp:"+JSON.stringify(result_resp));

        if(callback_id === init_callback_id){
            //验证成功
            if(result_resp.code == 0){
                _isValidated = true;
            }else{
                _isValidated = false;
            }
        }

        var result_code = result_resp.code;
        var callback = _callback_map[callback_id];
        if(callback && typeof callback === 'function'){
            if(result_code === 0){
                //成功
                var data = result_resp.data;
                if(!data){
                    result_resp.data={};
                }
            }else{
                //失败
                if(!result_resp.reason){
                    result_resp.reason="未知错误！";
                }
            }
            callback(result_resp);
        }
        delete _callback_map.callback_id;
    }

    function _set_debug_mode(mode){
        _isDebugMode = mode;
    }

    function _init(data,init_callback){
        if(!data){
            //初始化数据为空!
            if(init_callback && typeof init_callback === 'function' ){
                //回调函数不为空
                var failResult = {};
                failResult.code = -1;
                failResult.reason = "Initialized Failed!";
                init_callback(failResult);
            }
            return;
        }
        _call(INIT_METHOD,data,function(result){
            //初始化结果。
            if(result.code == 0){
                //成功！
                _isValidated = true;
            }else{
                _isValidated = false;
            }
            init_callback(result);
        });

    }


    var _YMMBridge = {
        VERSION: VERSION_NAME,
        VERSION_CODE: VERSION_CODE,
        //////////JS-CALL//////////
        //订阅事件
        subscribeEvent: _subscribe_event,
        //取消订阅事件
        unsubscribeEvent: _unsubscribe_event,
        //请求
        call: _call,
        //////////Native-CALL////////////
        //事件通知
        notifyEvent: _notify_event,
        //告知JS中间层，已经收到请求。可以发送下一条请求了
        callReceivedReply: _call_reply_from_native,
        //通知请求结果
        notifyResult:_notify_result,
        //设置Debug模式
        setDebugMode:_set_debug_mode,
        //初始化SDK
        init:_init
    };

    function _createQueueReadyIframe(doc) {
        // _setResultIframe 的初始化
//        _setResultIframe = doc.createElement('iframe');
//        _setResultIframe.id = '__YMMJSBridgeIframe_Request';
//        _setResultIframe.style.display = 'none';
//        doc.documentElement.appendChild(_setResultIframe);

        _readyMessageIframe = doc.createElement('iframe');
        _readyMessageIframe.id = '__YMMJSBridgeIframe';
        _readyMessageIframe.style.display = 'none';
        doc.documentElement.appendChild(_readyMessageIframe);
        return _readyMessageIframe;
    }

    window.YmmBridge = _YMMBridge;

    if(window.onBridgeReady && typeof window.onBridgeReady === 'function' ){
        window.onBridgeReady();
        delete window.onBridgeReady;
    }


    /////////////////////Utils////////////////////////
    //UUID
    function uuid() {
        var s = [];
        var hexDigits = "0123456789abcdef";
        for (var i = 0; i < 36; i++) {
            s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
        }
        s[14] = "4"; // bits 12-15 of 1the time_hi_and_version field to 0010
        s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
        s[8] = s[13] = s[18] = s[23] = "-";

        var uuid = s.join("");
        return uuid;
    }


})();
